#!/bin/bash -l
# This wrapper script is intended to be submitted to Grid Engine to support
# communicating jobs.
#
# Modified for Legion/Grace April 2018 - BAA/WH
# Updated for R2018b on Myriad/Legion/Grace January 2019
# Updated for R2019b on Myriad - December 2019.
# Updated for R2021a on Myriad - April 2021.
#
#$ -l mem=2G
#$ -l matlab=1
#$ -jsv /shared/ucl/apps/Matlab/toolbox_local/envset.jsv
#$ -cwd
#
# Ensure that under Grid Engine, we're in /bin/bash too:
#$ -S /bin/bash
#

# This script uses the following environment variables set by the submit MATLAB code:
# PARALLEL_SERVER_CMR         - the value of ClusterMatlabRoot (may be empty)
# PARALLEL_SERVER_MATLAB_EXE  - the MATLAB executable to use
# PARALLEL_SERVER_MATLAB_ARGS - the MATLAB args to use
# PARALLEL_SERVER_DEBUG       - used to debug problems on the cluster
#
# The following environment variables are forwarded through mpiexec:
# PARALLEL_SERVER_DECODE_FUNCTION     - the decode function to use
# PARALLEL_SERVER_STORAGE_LOCATION    - used by decode function
# PARALLEL_SERVER_STORAGE_CONSTRUCTOR - used by decode function
# PARALLEL_SERVER_JOB_LOCATION        - used by decode function
#
# The following environment variables are set by LSF:
# PE_HOSTFILE - list of hostnames with their associated number of processors allocated to this Grid Engine job

# Copyright 2006-2019 The MathWorks, Inc.

# If PARALLEL_SERVER_ environment variables are not set, assign any
# available values with form MDCE_ for backwards compatibility
PARALLEL_SERVER_CMR=${PARALLEL_SERVER_CMR:="${MDCE_CMR}"}
PARALLEL_SERVER_MATLAB_EXE=${PARALLEL_SERVER_MATLAB_EXE:="${MDCE_MATLAB_EXE}"}
PARALLEL_SERVER_MATLAB_ARGS=${PARALLEL_SERVER_MATLAB_ARGS:="${MDCE_MATLAB_ARGS}"}
PARALLEL_SERVER_DEBUG=${PARALLEL_SERVER_DEBUG:="${MDCE_DEBUG}"}

# Load the Matlab module on Kathleen/Myriad - need the Intel MPI module as well.

module load gcc-libs/4.9.2
module load compilers/intel/2018/update3
module load mpi/intel/2018/update3/intel
module load xorg-utils/X11R7.7
module load matlab/full/r2021a/9.10
module list
echo ""

# Echo the resources that the scheduler has allocated to this job:
echo -e "The scheduler has allocated the following resources to this job (format is [hostname] [number of parallel processes to run on host] [queue name] [processor range]):\n$(cat ${PE_HOSTFILE:?"Host file undefined"})"

# Create full path to mw_mpiexec if needed.
# FULL_MPIEXEC=${PARALLEL_SERVER_CMR:+${PARALLEL_SERVER_CMR}/bin/}mw_mpiexec
FULL_MPIEXEC=/shared/ucl/apps/intel-mpi/ucl-wrapper/bin/mpirun

# Label stdout/stderr with the rank of the process
MPI_VERBOSE=-l

# Increase the verbosity of mpiexec if PARALLEL_SERVER_DEBUG is true
if [ "X${PARALLEL_SERVER_DEBUG}X" = "XtrueX" ] ; then
    MPI_VERBOSE="${MPI_VERBOSE} -v -print-all-exitcodes"
fi

# Construct the command to run.
CMD="\"${FULL_MPIEXEC}\" ${MPI_VERBOSE} \"${PARALLEL_SERVER_MATLAB_EXE}\" ${PARALLEL_SERVER_MATLAB_ARGS}"

# Echo the command so that it is shown in the output log.
echo $CMD

# Execute the command.
eval $CMD

MPIEXEC_EXIT_CODE=${?}
if [ ${MPIEXEC_EXIT_CODE} -eq 42 ] ; then
    # Get here if user code errored out within MATLAB. Overwrite this to zero in
    # this case.
    echo "Overwriting MPIEXEC exit code from 42 to zero (42 indicates a user-code failure)"
    MPIEXEC_EXIT_CODE=0
fi
echo "Exiting with code: ${MPIEXEC_EXIT_CODE}"
exit ${MPIEXEC_EXIT_CODE}
